///////////////////////////////////////////////
//                                           //
// VRLab Mental Vision Module 10: Stereo     //
// Author: Achille Peternier                 //
// Copyright: EPFL-VRLab 2006 (C)            //
//                                           //
///////////////////////////////////////////////
#include "../data/common.h"
#pragma comment(lib, "opengl32.lib")



//////////////
// #DEFINES //
//////////////
#define MODULE_NAME "Stereo"
float zoom = 3.0f;



//////////
// MAIN //
//////////

int main(int argc, char *argv[])
{ 
   /////////////////
   // Init MVisio //
   /////////////////
   MVOPENGL::setNearPlane(0.1f);
   MVOPENGL::setFarPlane(100.0f);
   INIT_MVISIO(MODULE_NAME, argc, argv);   
   
   // Common interface:
   USE_FONT();  
   USE_BACKGROUND(MODULE_SIZEX, MODULE_SIZEY);
   USE_QUITBUTTON(MODULE_SIZEX, MODULE_SIZEY);    
   USE_CAMERA(0.0f, 1.0f, 0.0f, 0.0f, 0.0f, 0.0f); 
   USE_LIGHT(0.0f, 100.0f, 100.0f);    
   light->setSpecular(0.6f, 0.6f, 0.6f, 1.0f);    
   light->setPosition(0.0f, 2.0f, 0.0f);   
   camera->setTargetNode(NULL);   
   //camera->setRotation(-30.0f, 40.0f, 0.0f);
   
   
   /////////
   // Scene:
   MVNODE *scene = MVISIO::load("bar.MVE");
   scene->move(0.0f, 1.0f, 0.0f);
   scene->rotate(0.0f, -40.0f, 0.0f);
   
   //////////////////
   // 3D box display:   
   MVTEXTURE *rTex = new MVTEXTURE();
   rTex->setFilter(true, false);
   rTex->create(512, 512, MV_TEXTURE_32BIT);
   
   MVTEXTURE *lTex = new MVTEXTURE();
   lTex->setFilter(true, false);
   lTex->create(512, 512, MV_TEXTURE_32BIT);
   
   MVBOX *eyeRight = new MVBOX();
   eyeRight->setSizes((int) ((MODULE_SIZEY/1.22)*(4.0f/3.0f)), (int) (MODULE_SIZEY/1.22));
   eyeRight->setThickness(0);
   eyeRight->setPosition(0, (int) (MODULE_SIZEY/17.4));
   eyeRight->setTexture(rTex);
   eyeRight->setColor(1.0f, 0.0f, 0.0f, 1.0f);
   
   MVBOX *eyeLeft = new MVBOX();
   eyeLeft->setSizes((int) ((MODULE_SIZEY/1.22)*(4.0f/3.0f)), (int) (MODULE_SIZEY/1.22));
   eyeLeft->setThickness(0);
   eyeLeft->setPosition(0, (int) (MODULE_SIZEY/17.5));
   eyeLeft->setTexture(lTex);
   eyeLeft->setColor(0.0f, 0.0f, 1.0f, 0.5f);
   
   MVOPENGL::setBackgroundColor(0.0f, 0.0f, 0.0f, 1.0f);
  
   // Scrollbar:
   int centeredX = (int) (eyeRight->getSizeX()+3);
   float EYE_DIST = 0.1f;
   MVHSCROLLBAR *sc = new MVHSCROLLBAR();
   sc->setPosition(centeredX, MODULE_SIZEY/10.8);
   sc->setSizeX((MODULE_SIZEX-centeredX)-2);
   sc->setColor(1.0f, 1.0f, 1.0f, 1.0f);
   sc->getCursor()->setColor(1.0f, 1.0f, 1.0f, 1.0f);
   sc->setMaxValue(0.20f);
   sc->setMinValue(0.0f);
   sc->setValue(EYE_DIST);  
   
   // Window:
   MVWINDOW *win = new MVWINDOW();
   win->setFont(font);
   win->setSizes(512, 256);   
   win->setCaption("^x0100^c8338Left eye^x0210^c3388Right eye");
   
   MVBOX *monitorRight = new MVBOX();
   monitorRight->setSizes(256, 256);
   monitorRight->setTexture(rTex);
   monitorRight->setColor(1.0f, 1.0f, 1.0f, 1.0f);
   monitorRight->setPosition(0, 0);
   monitorRight->setThickness(2);
   monitorRight->link(win);
   
   MVBOX *monitorLeft = new MVBOX();
   monitorLeft->setSizes(256, 256);
   monitorLeft->setTexture(lTex);
   monitorLeft->setColor(1.0f, 1.0f, 1.0f, 1.0f);
   monitorLeft->setPosition(256, 0);
   monitorLeft->setThickness(2);
   monitorLeft->link(win); 
   
   win->setPosition(MODULE_SIZEX/2 - win->getSizeX()/2, MODULE_SIZEY/10);
   
   // Calibration sliders: 
   // Tex:
   MVTEXTURE *gla = new MVTEXTURE();
   gla->load("glasses.tga");   
   
   MVBOX *glasses = new MVBOX();
   glasses->setTexture(gla);
   glasses->setPosition(centeredX, MODULE_SIZEY/8.8);
   glasses->setSizes((MODULE_SIZEX-centeredX)-2, (MODULE_SIZEX-centeredX)/2);
   
   // Right:
   MVHSCROLLBAR *rR = new MVHSCROLLBAR();
   rR->setSizeX((MODULE_SIZEX-centeredX)-30);
   rR->setValue(0.9f);
   rR->setColor(1.0f, 0.0f, 0.0f, 1.0f);
   rR->getCursor()->setColor(1.0f, 0.0f, 0.0f, 1.0f);
   rR->setPosition(centeredX+16, MODULE_SIZEY/3+35);
   
   MVHSCROLLBAR *rG = new MVHSCROLLBAR();
   rG->setSizeX((MODULE_SIZEX-centeredX)-30);
   rG->setValue(0.0f);
   rG->setColor(0.0f, 1.0f, 0.0f, 1.0f);
   rG->getCursor()->setColor(0.0f, 1.0f, 0.0f, 1.0f);
   rG->setPosition(centeredX+16, MODULE_SIZEY/3+48);
  
   MVHSCROLLBAR *rB = new MVHSCROLLBAR();
   rB->setSizeX((MODULE_SIZEX-centeredX)-30);
   rB->setValue(0.0f);
   rB->setColor(0.0f, 0.0f, 1.0f, 1.0f);
   rB->getCursor()->setColor(0.0f, 0.0f, 1.0f, 1.0f);
   rB->setPosition(centeredX+16, MODULE_SIZEY/3+61);  
   
   // Left
   MVHSCROLLBAR *lR = new MVHSCROLLBAR();
   lR->setSizeX((MODULE_SIZEX-centeredX)-30);
   lR->setValue(0.0f);
   lR->setColor(1.0f, 0.0f, 0.0f, 1.0f);
   lR->getCursor()->setColor(1.0f, 0.0f, 0.0f, 1.0f);
   lR->setPosition(centeredX+16, MODULE_SIZEY/2.2+20);
   
   MVHSCROLLBAR *lG = new MVHSCROLLBAR();
   lG->setSizeX((MODULE_SIZEX-centeredX)-30);
   lG->setValue(0.8f);
   lG->setColor(0.0f, 1.0f, 0.0f, 1.0f);
   lG->getCursor()->setColor(0.0f, 1.0f, 0.0f, 1.0f);
   lG->setPosition(centeredX+16, MODULE_SIZEY/2.2+35);
  
   MVHSCROLLBAR *lB = new MVHSCROLLBAR();
   lB->setSizeX((MODULE_SIZEX-centeredX)-30);
   lB->setValue(1.0f);
   lB->setColor(0.0f, 0.0f, 1.0f, 1.0f);
   lB->getCursor()->setColor(0.0f, 0.0f, 1.0f, 1.0f);
   lB->setPosition(centeredX+16, MODULE_SIZEY/2.2+46);     
   
   // Calibration colors:   
   MVBOX *calBlack = new MVBOX();
   calBlack->setColor(0.0f, 0.0f, 0.0f, 1.0f);
   calBlack->setSizes(75, 74);
   calBlack->setThickness(0);
   calBlack->setPosition(centeredX+(MODULE_SIZEX-centeredX)/2-calBlack->getSizeX()/2, MODULE_SIZEY/4-5);
   
   MVBOX *calRed = new MVBOX();
   calRed->setColor(1.0f, 0.0f, 0.0f, 1.0f);
   calRed->setSizes(64, 64);
   calRed->setThickness(0);
   calRed->setPosition(centeredX+(MODULE_SIZEX-centeredX)/2-calRed->getSizeX()/2, MODULE_SIZEY/4);
   
   MVBOX *calBlue = new MVBOX();
   calBlue->setColor(0.0f, 0.0f, 1.0f, 1.0f);
   calBlue->setSizes(32, 32);
   calBlue->setThickness(0);
   calBlue->setPosition(centeredX+(MODULE_SIZEX-centeredX)/2-calBlue->getSizeX()/2, MODULE_SIZEY/4+calRed->getSizeY()/4);   
  
   MVOPENGL::cullingOff();
  
///////////////
// Mainloop: //
///////////////
   
   bool done = false;
   int mouseX = 0, mouseY = 0, oldMouseX = 0, oldMouseY = 0;   
   bool mouseBL = false, mouseBR = false;
   bool oldMouseBL = false, oldMouseBR = false;  
   
   while (!done)   
	{			      
	      // Update values:
	      camera->setPosition(0.0f, 1.0f, zoom);
	      calRed->setColor(rR->getValue(), rG->getValue(), rB->getValue(), 1.0f);
	      eyeRight->setColor(calRed->getColor());	      
	      calBlue->setColor(lR->getValue(), lG->getValue(), lB->getValue(), 1.0f);
	      eyeLeft->setColor(calBlue->getColor().r, calBlue->getColor().g, calBlue->getColor().b, 0.5f);	      
	      EYE_DIST = sc->getValue();
	
	      // Right eye:
         camera->moveDoom(-EYE_DIST, 0.0f, 0.0f);
         glViewport(0, 0, 512, 512);
         MVISIO::clear(true, true);         
         MVISIO::begin3D(camera);     
            light->pass();       
            scene->pass();
         MVISIO::end3D();
         glBindTexture(GL_TEXTURE_2D, rTex->getGLId());
         glCopyTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 0, 0, 512, 512, 0);
         //MVELEMENT::getFBO3()->unbindFromFBO();
         
         // Left eye:
         camera->moveDoom(+EYE_DIST*2, 0.0f, 0.0f);
         //MVELEMENT::getFBO4()->bindToFBOAux2();
         MVISIO::clear(true, true);         
         MVISIO::begin3D(camera);  
            light->pass();          
            scene->pass();
         MVISIO::end3D();
         glBindTexture(GL_TEXTURE_2D, lTex->getGLId());
         glCopyTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 0, 0, 512, 512, 0); 
         //MVELEMENT::getFBO4()->unbindFromFBO();
         
         // Center view:
         MVOPENGL::resize(0, 0, MODULE_SIZEX, MODULE_SIZEY);
         camera->moveDoom(-EYE_DIST, 0.0f, 0.0f);     
             
	      ////////////////////////
	      // Rendering background:	   
	   	MVISIO::clear(true, true);	   		   	
			MVISIO::begin2D();			   
			   PASS_BACKGROUND;					     			  
			MVISIO::end2D();
			
			
			///////////////////
			// Rendering boxes:
			MVISIO::begin2D();			
            glBlendFunc(GL_ONE, GL_ONE);
            eyeRight->pass();
            eyeLeft->pass();                     
			MVISIO::end2D();								
			
			
			////////////////////////
			// Rendering foreground:			
			MVISIO::begin2D();		
			   glLineWidth(LINE_WIDTH);		   
			   MVOPENGL::blendingOff();
			   MVTEXTURE::texturingOff();		   
			   glDisable(GL_BLEND);
			   glDisable(GL_TEXTURE_2D);
			   glBegin(GL_LINES);			      
			      glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
			      glVertex2i(eyeRight->getSizeX(), MODULE_SIZEY/17.5);				      
			      glVertex2i(eyeRight->getSizeX(), MODULE_SIZEY/17.5 + eyeRight->getSizeY()+1);
			   glEnd();		 
			   
			   // Title:
			   font->putString(10, 13, MODULE_NAME);	 
            
			   // Interface:
			   PASS_QUITBUTTON;	
			   
			   // Calibration:
			   win->pass();	
			   
			   font->putString(centeredX, MODULE_SIZEY/14.0, "Eye distance: %.3f cm", sc->getValue());
			   sc->pass();	
			   glasses->pass();				   
			   font->putString(centeredX, MODULE_SIZEY/3.0+15, "Right eye color correction:");
			   font->putString(centeredX, MODULE_SIZEY/2.2, "Left eye color correction:");
			   calBlack->pass();
			   calRed->pass();
			   calBlue->pass();	
			   rR->pass();   		
			   rG->pass();
			   rB->pass();	   		   
			   lR->pass();   		
			   lG->pass();
			   lB->pass();	  			  
			MVISIO::end2D();
			
		   // Update mouse:		   
			oldMouseBR = mouseBR;
			oldMouseBL = mouseBL;
			int buttonState = SDL_GetMouseState(&mouseX, &mouseY);		
         if (buttonState&SDL_BUTTON(1))
					mouseBL = true;					
			else 
					mouseBL = false;
         if (buttonState&SDL_BUTTON(2) || buttonState&SDL_BUTTON(3))
					mouseBR = true;					
			else 
					mouseBR = false;						
      					
			
			///////////////////////
			// Check MVisio events:
			MVID id=0;
			bool used =	MVELEMENT::manageGUIEvents(0, mouseX, mouseY, mouseBL, mouseBR, &id);  
			if (PRESSED_QUITBUTTON(id))
			      done = true;         
			
			int RmouseX, RmouseY;
			SDL_GetRelativeMouseState(&RmouseX, &RmouseY);
			if (mouseX < centeredX && !used && mouseBL && !mouseBR)
			{			      
			      scene->rotate(RmouseY/4.0f, RmouseX/4.0f, 0.0f);
			      if (scene->getRotation().x < -80.0f) scene->setRotation(-80.0f, scene->getRotation().y, 0.0f);
			      if (scene->getRotation().x > 80.0f) scene->setRotation(80.0f, scene->getRotation().y, 0.0f);
			}
			if (mouseBL && mouseBR && !used && mouseX < centeredX)
			{
			      zoom += (float) (RmouseY)/16.0f;
			      if (zoom <1.0f) zoom = 1.0f;
			      if (zoom >30.0f) zoom = 30.0f;
			}
			
			// Update keyboard (like a joypad):
			Uint8 *array;
			int arraySize;
			array = SDL_GetKeyState(&arraySize);
			if (array[SDLK_PAGEUP])    
			{
               zoom += 2.0f * MVISIO::getFPSfactor();
               if (zoom > 30.0f)
			            zoom = 30.0f;			             
         }							       								                     
         else
               if (array[SDLK_PAGEDOWN])
               {
                     zoom -= 2.0f * MVISIO::getFPSfactor();
                       if (zoom < 1.0f)
			                  zoom = 1.0f;                                          
               }							
			
			////////////////
			// Check events:			
			SDL_Event _event;			
			//SDL_PumpEvents();						
			while (SDL_PollEvent(&_event)) 
			{
					// Quit by click on X:
					if (_event.type == SDL_QUIT) 
							done = true;	
					
					// Resize?
				   if (_event.type == SDL_VIDEORESIZE)
				         MVSETTINGS::resize(0, 0, _event.resize.w, _event.resize.h);							
               
               // Quit by pressing ESC:
					if (_event.type == SDL_KEYDOWN) 
					{
							// Quit by ESC:
							if (_event.key.keysym.sym == SDLK_ESCAPE) 
							       done = true;											
			      }															
			}     
			
			// Wait and swap:			
			Sleep(5); // Don't kill mthreading ;)
			MVISIO::swap();					
 	}      
   
   // Done:   
   MVISIO::free();
   return 0;
}
